@@ -212,177 +212,6 @@ sub asinh {
   return $e2;
 }
 
-sub tanh {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    if ($e1->im->is_infinite) {
-      $e2 = complex_double(NaN, NaN);
-    }
-    elsif ($e1->re->is_positive_infinite) {
-      $e2 = complex(1, 0);
-    }
-    elsif ($e1->re->is_negative_infinite) {
-      $e2 = complex(-1, 0);
-    }
-    else {
-      $e2 = divide(sinh($e1), cosh($e1));
-    }
-  }
-  elsif ($e1->is_numeric || $e1->is_logical) {
-    $e1 = $e1->as_double;
-    
-    return $e1 if $e1->is_nan;
-    
-    if ($e1->is_positive_infinite) {
-      $e2 = double(1);
-    }
-    elsif ($e1->is_negative_infinite) {
-      $e2 = double(-1);
-    }
-    else {
-      $e2 = divide(sinh($e1), cosh($e1));
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
-sub cosh {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    my $e1_x = $e1->re;
-    my $e1_y = $e1->im;
-    
-    my $e2_cy = Rstats::ElementsFunc::cos($e1_y);
-    my $e2_sy = Rstats::ElementsFunc::sin($e1_y);
-    my $e2_ex = Rstats::ElementsFunc::exp($e1_x);
-    
-    my $e2_ex_1 = not_equal($e2_ex, double(0)) ? divide(double(1), $e2_ex) : Inf;
-
-    my $e2_x = divide(
-      multiply(
-        $e2_cy,
-        add($e2_ex, $e2_ex_1)
-      ),
-      double(2)
-    );
-    my $e2_y = divide(
-      multiply(
-        $e2_sy,
-        subtract($e2_ex, $e2_ex_1)
-      ),
-      double(2)
-    );
-    
-    $e2 = complex_double($e2_x, $e2_y);
-  }
-  elsif ($e1->is_numeric || $e1->is_logical) {
-    $e1 = $e1->as_double;
-    return $e1 if $e1->is_nan;
-    
-    my $e2_ex = Rstats::ElementsFunc::exp($e1);
-    
-    if (not_equal($e2_ex, double(0))) {
-      if ($e2_ex->is_positive_infinite) {
-        $e2 = Inf;
-      }
-      else {
-        $e2 = divide(
-          add(
-            $e2_ex,
-            divide(double(1), $e2_ex)
-          ),
-          double(2)
-        )
-      }
-    }
-    else {
-      $e2 = Inf;
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
-sub sinh {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    my $e1_x = $e1->re;
-    my $e1_y = $e1->im;
-    
-    my $e2_cy = Rstats::ElementsFunc::cos($e1_y);
-    my $e2_sy = Rstats::ElementsFunc::sin($e1_y);
-    my $e2_ex = Rstats::ElementsFunc::exp($e1_x);
-    
-    my $e2_ex_1 = not_equal($e2_ex, double(0)) ? divide(double(1), $e2_ex) : Inf;
-
-    my $e2_x = divide(
-      multiply(
-        $e2_cy,
-        subtract($e2_ex, $e2_ex_1)
-      ),
-      double(2)
-    );
-    my $e2_y = divide(
-      multiply(
-        $e2_sy,
-        add($e2_ex, $e2_ex_1)
-      ),
-      double(2)
-    );
-    
-    $e2 = complex_double($e2_x, $e2_y);
-  }
-  elsif ($e1->is_numeric || $e1->is_logical) {
-    $e1 = $e1->as_double;
-    return $e1 if $e1->is_nan;
-    
-    return double(0) if equal($e1, double(0));
-    my $e2_ex = Rstats::ElementsFunc::exp($e1);
-    
-    if (not_equal($e2_ex, double(0))) {
-      if ($e2_ex->is_positive_infinite) {
-        $e2 = Inf;
-      }
-      else {
-        $e2 = divide(
-          subtract(
-            $e2_ex,
-            divide(double(1), $e2_ex)
-          ),
-          double(2)
-        )
-      }
-    }
-    else {
-      $e2 = negativeInf;
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
 sub atan {
   my $e1 = shift;
   
@@ -507,96 +336,6 @@ sub atan2 {
   return $e3;
 }
 
-sub log2 {
-  my $e1 = shift;
-  
-  my $e2 = divide(
-    Rstats::ElementsFunc::log($e1),
-    $e1->is_complex ? Rstats::ElementsFunc::log(complex(2, 0)) : Rstats::ElementsFunc::log(double(2))
-  );
-  
-  return $e2;
-}
-
-sub log10 {
-  my $e1 = shift;
-  
-  my $e2 = divide(
-    Rstats::ElementsFunc::log($e1),
-    $e1->is_complex ? Rstats::ElementsFunc::log(complex(10, 0)) : Rstats::ElementsFunc::log(double(10))
-  );
-  
-  return $e2;
-}
-
-sub log {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    
-    my $e1_r = Mod($e1);
-    my $e1_t = Arg($e1);
-    
-    if (equal($e1_r, double(0))) {
-      $e2 = negativeInf;
-    }
-    else {
-      if (more_than($e1_t, pi)) {
-        $e1_t = subtract(
-          $e1_t,
-          multiply(
-            pi,
-            pi
-          )
-        )
-      }
-      elsif (less_than_or_equal($e1_t, negation(pi))) {
-        $e1_t = add(
-          $e1_t,
-          multiply(
-            pi,
-            pi
-          )
-        )
-      }
-      
-      $e2 = complex_double(Rstats::ElementsFunc::log($e1_r), $e1_t);
-    }
-  }
-  elsif ($e1->is_numeric || $e1->is_logical) {
-    $e1 = $e1->as_double unless $e1->is_double;
-    my $value = $e1->dv;
-    
-    if ($e1->is_infinite) {
-      carp "In cos : NaNs produced";
-      $e2 = NaN;
-    }
-    elsif ($e1->is_nan) {
-      $e2 = $e1;
-    }
-    else {
-      if ($value < 0) {
-        carp "In log : NaNs produced";
-        $e2 = NaN
-      }
-      elsif ($value == 0) {
-        $e2 = negativeInf;
-      }
-      else {
-        $e2 = double(log($value));
-      }
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
 sub Mod { Rstats::ElementsFunc::abs(@_) }
 
 sub Arg {
@@ -631,126 +370,6 @@ sub hash {
   return $hash;
 }
 
-sub tan {
-  my $e1 = shift;
-  
-  my $e2 = divide(Rstats::ElementsFunc::sin($e1), Rstats::ElementsFunc::cos($e1));
-  
-  return $e2;
-}
-
-sub cos {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    
-    my $e1_re = $e1->re;
-    my $e1_im = $e1->im;
-    
-    my $e2_eim = Rstats::ElementsFunc::exp($e1_im);
-    my $e2_sre = Rstats::ElementsFunc::sin($e1_re);
-    my $e2_cre = Rstats::ElementsFunc::cos($e1_re);
-    
-    my $e2_eim_1 = divide(double(1), $e2_eim);
-    
-    my $e2_re = divide(
-      multiply(
-        $e2_cre,
-        add(
-          $e2_eim,
-          $e2_eim_1
-        )
-      ),
-      double(2)
-    );
-    
-    my $e2_im = divide(
-      multiply(
-        $e2_sre,
-        subtract(
-          $e2_eim_1,
-          $e2_eim
-        )
-      ),
-      double(2)
-    );
-    
-    $e2 = complex_double($e2_re, $e2_im);
-  }
-  elsif ($e1->is_numeric || $e1->is_logical) {
-    $e1 = $e1->as_double unless $e1->is_double;
-    my $value = $e1->dv;
-    
-    if ($e1->is_infinite) {
-      carp "In cos : NaNs produced";
-      $e2 = NaN;
-    }
-    elsif ($e1->is_nan) {
-      $e2 = $e1;
-    }
-    else {
-      $e2 = double(cos($value));
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
-sub exp {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    
-    my $e1_re = $e1->re;
-    my $e1_im = $e1->im;
-    
-    my $e2_mod = Rstats::ElementsFunc::exp($e1_re);
-    my $e2_arg = $e1_im;
-
-    my $e2_re = Rstats::ElementsFunc::multiply(
-      $e2_mod,
-      Rstats::ElementsFunc::cos($e2_arg)
-    );
-    my $e2_im = Rstats::ElementsFunc::multiply(
-      $e2_mod,
-      Rstats::ElementsFunc::sin($e2_arg)
-    );
-    
-    $e2 = complex_double($e2_re, $e2_im);
-  }
-  elsif ($e1->is_double || $e1->is_integer || $e1->is_logical) {
-    $e1 = $e1->as_double unless $e1->is_double;
-    
-    if ($e1->is_positive_infinite) {
-      $e2 = Inf;
-    }
-    elsif ($e1->is_negative_infinite) {
-      $e2 = double(0);
-    }
-    elsif ($e1->is_nan) {
-      $e2 = $e1;
-    }
-    else {
-      my $value = $e1->dv;
-      $e2 = double(exp($value));
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
 sub expm1 {
   my $e1 = shift;
   
@@ -1024,69 +643,6 @@ sub asin {
   return $e2;
 }
 
-sub sin {
-  my $e1 = shift;
-  
-  return $e1 if $e1->is_na;
-  
-  my $e2;
-  if ($e1->is_complex) {
-    
-    my $e1_re = $e1->re;
-    my $e1_im = $e1->im;
-    
-    my $e2_eim = Rstats::ElementsFunc::exp($e1_im);
-    my $e2_sre = Rstats::ElementsFunc::sin($e1_re);
-    my $e2_cre = Rstats::ElementsFunc::cos($e1_re);
-    
-    my $e2_eim_1 = divide(double(1), $e2_eim);
-    
-    my $e2_re = divide(
-      multiply(
-        $e2_sre,
-        add(
-          $e2_eim,
-          $e2_eim_1
-        )
-      ),
-      double(2)
-    );
-    
-    my $e2_im = divide(
-      multiply(
-        $e2_cre,
-        subtract(
-          $e2_eim,
-          $e2_eim_1
-        )
-      ),
-      double(2)
-    );
-    
-    $e2 = complex_double($e2_re, $e2_im);
-  }
-  elsif ($e1->is_numeric || $e1->is_logical) {
-    $e1 = $e1->as_double unless $e1->is_double;
-    my $value = $e1->dv;
-    
-    if ($e1->is_infinite) {
-      carp "In sin : NaNs produced";
-      $e2 = NaN;
-    }
-    elsif ($e1->is_nan) {
-      $e2 = $e1;
-    }
-    else {
-      $e2 = double(sin($value));
-    }
-  }
-  else {
-    croak "Not implemented";
-  }
-  
-  return $e2;
-}
-
 sub create {
   my ($type, $value) = @_;
   
@@ -1265,47 +821,6 @@ sub Im {
   }
 }
 
-sub abs {
-  my $e1 = shift;
-  
-  if ($e1->is_complex) {
-    return raise(add(raise($e1->re, double(2)), raise($e1->im, double(2))), double(1/2));
-  }
-  elsif ($e1->is_double || $e1->is_integer) {
-    my $type = $e1->type;
-    my $zero = Rstats::ElementsFunc::create($type);
-    if (more_than($e1, $zero)) {
-      return $e1;
-    }
-    else {
-      return negation($e1);
-    }
-  }
-  elsif ($e1->is_logical) {
-    my $zero = Rstats::ElementsFunc::create('logical');
-    if (more_than($e1, $zero)) {
-      return logical_to_integer($e1);
-    }
-    else {
-      return negation(logical_to_integer($e1));
-    }
-  }
-  else {
-    croak 'non-numeric argument to mathematical function';
-  }
-}
-
-sub logical_to_integer {
-  my $e1 = shift;
-  
-  if ($e1->is_logical) {
-    return integer($e1->iv);
-  }
-  else {
-    return $e1;
-  }
-}
-
 sub more_than {
   my ($e1, $e2) = @_;
   
